      real function resyn( t, u )
c  Copyright (C) 2015, Jerry Sellwood
c
c routine to reconstruct the disk surface density from its discrete Fourier
c    transform.  It also adds a projected bulge component if bulge = .T.
      use aarrays
      implicit none
c
c calling arguments
      real t, u
c
c common blocks
c
      include 'inc/params.f'
c
      include 'inc/admin.f'
c
      include 'inc/anlys.f'
c
      include 'inc/grids.f'
c
c externals
      real grofu, sbulge
c
c local variables
      integer i, iu, j, last, m, mm1, n
      save last
      real c, r, th, thfc, w, w1, w2
c
      data last / -1000 /
c
      if( .not. ( p2d .or. p3d ) )call crash( 'RESYN',
     +                                            'Inappropriate grid' )
      if( istep .ne. last )then
c take out normalizing factor just once each time step
        n = 1
        do iu = 1, mr
          do i = 1, ma - 1
c NB wres( n ) is the surface density
c   if I want masses, I should also correct for area
            wres( n + i ) = wres( n + i ) * wres( n )
            m = n + i + mr * ma
            wres( m ) = wres( m ) * wres( n )
          end do
          n = n + ma
        end do
        last = istep
      end if
c interpolate between grid rings
      iu = u
      iu = min( iu, nr( jgrid ) - 1 )
      w = u - real( iu )
      n = ma * iu + 1
      resyn = 0
      if( wres( n ) .ne. 0. )then
c non-axisymmetric part
        m = n + mr * ma
        thfc = alpha * ( t - 1. )
        mm1 = max( mml / nsect, 1 )
        do i = mm1, mmu / nsect
          th = thfc * real( i * nsect )
c linear interpolation between grid rings
          j = n + i
          c = ( 1. - w ) * wres( j ) + w * wres( j + ma )
          resyn = resyn + c * cos( th )
          j = m + i
          c = ( 1. - w ) * wres( j ) + w * wres( j + ma )
          resyn = resyn + c * sin( th )
        end do
c axisymmetric part
        if( mml .eq. 0 )then
          c = ( 1. - w ) * wres( n ) + w * wres( n + ma )
c why divide by the na factor?
          resyn = ( c + resyn ) / real( na )
          if( bulge )then
            r = grofu( u - .1 ) / lscale
            resyn = resyn + sbulge( r )
          end if
c normalise by undisturbed mass density
c          r = exp( alpha * ( u - 1. ) ) / lscale
c          den = gsigma( r ) * real( na )
c          resyn = resyn * wres( n ) / den
        end if
      end if
      return
      end
